#pragma once
#include "bridge_proxy.h"
#include <iostream>
#include "communication/bridge/proxy/i_bridge_proxy.h"
#include "domain/program.h"
#include "domain/group.h"

namespace spin
{

    class FakeBridgeProxy : public IBridgeProxy
    {
    public:
        FakeBridgeProxy() {}

        void connectTo(const std::string &host) const override
        {
            std::cout << "FakeBridgeProxy::connectTo called with host: " << host << std::endl;
        }

        void disconnect() const override
        {
            std::cout << "FakeBridgeProxy::disconnect called" << std::endl;
        }

        void connectFromCommandTo(const std::string &host) const override
        {
            std::cout << "FakeBridgeProxy::connectFromCommandTo called with host: " << host << std::endl;
        }

        void enableExhibitionMode() const override
        {
            std::cout << "FakeBridgeProxy::enableExhibitionMode called" << std::endl;
        }

        void acknowledgeErrors() const override
        {
            std::cout << "FakeBridgeProxy::acknowledgeErrors called" << std::endl;
        }

        void stopMotor() const override
        {
            std::cout << "FakeBridgeProxy::stopMotor called" << std::endl;
        }

        void startMotor(const int speed_rpm, const int rampTime_ms) const override
        {
            std::cout << "FakeBridgeProxy::startMotor called with speed: " << speed_rpm << " rpm" << std::endl;
        }

        void stopSafetyShield(const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::stopSafetyShield called with blocking: " << std::boolalpha << blocking << std::endl;
        }

        void doSafetyShieldHoming(const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::doSafetyShieldHoming called with blocking: " << std::boolalpha << blocking << std::endl;
        }

        void lockBit(const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::lockBit called with blocking: " << std::boolalpha << blocking << std::endl;
        }

        void releaseBit(const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::releaseBit called with blocking: " << std::boolalpha << blocking << std::endl;
        }

        void coverBit(const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::coverBit called with blocking: " << std::boolalpha << blocking << std::endl;
        }

        void coverScrew(const double &length, const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::coverScrew called with length: " << length << ", blocking: " << std::boolalpha << blocking << std::endl;
        }

        void moveSafetyShieldTo(const Length &length, const bool blocking = true) const override
        {
            std::cout << "FakeBridgeProxy::moveSafetyShieldTo called with length: " << length << ", blocking: " << std::boolalpha << blocking << std::endl;
        }

        void followRobot() const override
        {
            std::cout << "FakeBridgeProxy::followRobot called" << std::endl;
        }

        UUID startScrewProgram(const UUID &programUuid) const override
        {
            std::cout << "FakeBridgeProxy::startScrewProgram called with uuid: " << programUuid << std::endl;
            std::cout << "FakeBridgeProxy::startScrewProgram returning same uuid: " << programUuid << std::endl;
            return programUuid;

        }

        std::string programMessages() const override
        {
            std::cout << "FakeBridgeProxy::programMessages called" << std::endl;
            return "Fake program messages";
        }

        std::set<Program> programs() const override
        {
            std::cout << "FakeBridgeProxy::programs called" << std::endl;
            std::set<Program> fakePrograms;

            fakePrograms.insert(Program(uuid1, "Program 1", Length::fromMillimeters(16), Length::fromMillimeters(-6), Length::fromMillimeters(16
            ), true));
            fakePrograms.insert(Program(uuid2, "Program 2", Length::fromMillimeters(16), Length::fromMillimeters(-6), Length::fromMillimeters(16
            ), true));
            fakePrograms.insert(Program(uuid3, "Program 3", Length::fromMillimeters(16), Length::fromMillimeters(-6), Length::fromMillimeters(16
            ), true));

            return fakePrograms;
        }

        std::set<Group> groups() const override
        {
            std::cout << "FakeBridgeProxy::groups called" << std::endl;
            std::set<Group> fakeGroups;
            fakeGroups.insert(Group("Group 1", boost::uuids::string_generator()("00000000-0000-0000-0000-000000000011"), std::set<UUID>{uuid1, uuid2}));
            fakeGroups.insert(Group("Group 2", boost::uuids::string_generator()("00000000-0000-0000-0000-000000000012"), std::set<UUID>{uuid2, uuid3}));

            return fakeGroups;
        }

        std::optional<BridgeInfo> bridgeInfo() const override
        {
            std::cout << "FakeBridgeProxy::bridgeInfo called" << std::endl;
            return BridgeInfo(SerialNumber::parse("123456789012"), SoftwareVersion::parse("9.9.9"));
        }

        std::optional<ToolInfo> toolInfo() const override
        {
            std::cout << "FakeBridgeProxy::toolInfo called" << std::endl;
            return ToolInfo(SerialNumber::parse("123456789012"), SoftwareVersion::parse("9.9.9"), 15, 350);
        }

        Speed drivingSpeedSetpoint() const override
        {
            std::cout << "FakeBridgeProxy::drivingSpeedSetpoint called" << std::endl;
            return Speed::fromMillimetersPerSecond(15);
        }

        UUID latestResultUUID() const override
        {
            std::cout << "FakeBridgeProxy::latestResultUUID called" << std::endl;
            return uuid1;
        }

        ProgramResult::Evaluation latestResultEvaluation() const override
        {
            std::cout << "FakeBridgeProxy::latestResultEvaluation called" << std::endl;
            return ProgramResult::Evaluation::OK;
        }

    private:
        UUID uuid1 = boost::uuids::string_generator()("00000000-0000-0000-0000-000000000001");
        UUID uuid2 = boost::uuids::string_generator()("00000000-0000-0000-0000-000000000002");
        UUID uuid3 = boost::uuids::string_generator()("00000000-0000-0000-0000-000000000003");
    };
}